/* eslint-disable no-useless-constructor */
import React, { useState, useEffect, useRef } from 'react';
import { Input, Button, Tree, notification, Spin, message } from 'antd';
import { connect } from 'react-redux';
import classnames from 'classnames';

import { appService } from '@/api';
import { useHistory } from '@wisdom-utils/runtime';
import { savePagePartInfo } from '@/api/service/base';
import styles from './index.less';
import thumbnail from '../../assets/images/commonMenu/常用菜单.png';
import pageLogo from '../../assets/images/commonMenu/page-logo.png';
import starIcon from '../../assets/images/commonMenu/矢量智能对象 拷贝 3@2x.png';

// 是否是灰色的图标（灰色图标在白色背景中看不见，添加滤镜变色）
const isNeedFilterIcon = (icon = '') => {
  return !icon.includes('一级') && !icon.includes('ios/');
};

const CommonMenu = props => {
  const history = useHistory();
  const { menus } = props;
  const [commonMenus, setCommonMenus] = useState([]); // 常用菜单信息
  const [menuList, setMenuList] = useState([]);
  const [showMenuInfo, setShowMenuInfo] = useState(false); // 菜单列表显示隐藏
  const [loading, setLoading] = useState(true); // loading显示隐藏
  const [searchInfo, setSearchInfo] = useState('');
  const page = useRef(null);
  const searchBox = useRef(null);

  const defaultSubmitMenu = () => {
    let submitData = [];
    submitData = commonMenus.map(item => ({
      PartID: item.menuID,
      PartName: item.menuName,
      PartUrl: item.menuUrl,
      icon: item.menuIcon,
    }));
    return submitData;
  };

  /**
   * 添加、删除菜单
   * @param {string} opr add/delete，添加/删除
   * @param {object} extData { PartID, label, icon, url, product }，操作的菜单信息widget
   */
  const updateCommonMenu = (opr, node) => {
    const { href, extData } = node;
    const { PartID, label, icon, url, product } = extData;
    setLoading(true);
    let submitData = defaultSubmitMenu();
    if (opr === 'add') {
      submitData.push({
        PartID: `${PartID}`,
        PartName: label,
        PartUrl: href,
        icon,
      });
    } else if (opr === 'delete') {
      submitData = submitData.filter(item => item.PartID !== `${PartID}`);
    }
    savePagePartInfo({
      query: {
        UserID: window.globalConfig?.userInfo?.OID ?? '',
      },
      data: submitData,
    }).then(res => {
      if (res.statusCode === '0000') {
        message.success('修改常用菜单成功!');
        fetchMenus();
      } else {
        message.error('修改常用菜单失败!');
      }
    });
  };

  /**
   * 渲染自定义树节点
   * @param {object} node 树节点信息，至少包含key, title, extData, children
   * @returns {ReactDOM} 返回自定义节点DOM
   */
  const renderTitle = node => {
    const { key, title, extData = {}, children } = node;
    const { icon, url, isAdded } = extData;
    return (
      <div className={styles.customTitle}>
        <div
          className={classnames(
            styles.titleInfo,
            isNeedFilterIcon(icon) ? styles.filterIconBox : '',
          )}
        >
          <span className={styles.iconBox}>
            <img src={icon} alt="" />
          </span>
          <span>{title}</span>
        </div>
        {url && (
          <div className={styles.titleControl}>
            {isAdded ? (
              <div>
                <p className={styles.chooseLabel}>已添加</p>
                <div
                  className={styles.chooseBtn}
                  onClick={e => {
                    e.stopPropagation();
                    updateCommonMenu('delete', node);
                  }}
                >
                  取消添加
                </div>
              </div>
            ) : (
              <div>
                <div
                  className={styles.chooseBtn}
                  onClick={e => {
                    e.stopPropagation();
                    updateCommonMenu('add', node);
                  }}
                >
                  添加
                </div>
              </div>
            )}
          </div>
        )}
      </div>
    );
  };

  const fetchMenus = () => {
    appService
      .getPagePartInfo({
        UserID: window?.globalConfig?.userInfo?.OID ?? '',
      })
      .then(res => {
        setLoading(false);
        if (res.say.statusCode !== '0000')
          return notification.error({
            message: '服务错误',
            description: res.say.errMsg,
          });
        const newMenus = [];
        // 过滤出当前client的菜单
        const data = res.getMe.filter(item => {
          const client = item.PartID.split('_')[1] ?? '';
          return client === window.globalConfig.client;
        });
        data.forEach(item => {
          const newMenu = {
            menuIcon: '',
            menuName: '',
            menuGroup: '',
            menuID: '',
            menuUrl: '',
            menuPic: '',
          };
          newMenu.menuIcon = item.icon;
          newMenu.menuName = item.PartName;
          const PartIDArr = item.PartID.split('_');
          newMenu.menuGroup = PartIDArr.splice(2, PartIDArr.length - 3).join(
            '_',
          );
          newMenu.menuID = item.PartID;
          newMenu.menuUrl = item.PartUrl;
          newMenu.menuPic = item.BgPicUrl;
          newMenus.push(newMenu);
        });
        setCommonMenus(newMenus);
        // const w = getWidgets(newMenus);
        // setWidgets(w);
      })
      .catch(error => {
        notification.error({
          message: '服务错误',
          description: error,
        });
      });
  };

  useEffect(() => {
    fetchMenus();
  }, []);

  useEffect(() => {
    const isAddedMenu = menu => {
      const menuTmp = commonMenus.find(
        item => item.menuID === menu.extData.PartID,
      );
      return !!menuTmp;
    };
    const loop = (data, prePartID) => {
      return data.map(item => {
        const { name, level, href, key, path, routes } = item;
        const newmenu = { ...item };
        newmenu.key = href ? key : path;
        newmenu.title = name;

        // 菜单进行搜索过滤
        const index = name.indexOf(searchInfo);
        const beforeStr = name.substr(0, index);
        const afterStr = name.substr(index + searchInfo.length);
        const title =
          index > -1 ? (
            <span>
              {beforeStr}
              <span className={styles.treeSearchInfo}>{searchInfo}</span>
              {afterStr}
            </span>
          ) : (
            <span>{name}</span>
          );
        newmenu.title = title;
        newmenu.extData = {
          ...item.extData,
          PartID: `${prePartID}_${name}`,
        };
        newmenu.extData.isAdded = isAddedMenu(newmenu);

        if (routes) {
          newmenu.children = loop(routes, `${prePartID}_${name}`);
        }
        return newmenu;
      });
    };
    const newmenus = loop(menus, `widget_${window.globalConfig.client}`);
    setMenuList(newmenus);
  }, [menus, searchInfo, commonMenus]);

  const loop = data => {
    return data.map(item => {
      const index = item.name.indexOf(searchInfo);
      const beforeStr = item.name.substr(0, index);
      const afterStr = item.name.substr(index + searchInfo.length);
      const title =
        index > -1 ? (
          <span>
            {beforeStr}
            <span className={styles.treeSearchInfo}>{searchInfo}</span>
            {afterStr}
          </span>
        ) : (
          <span>{item.name}</span>
        );
      if (item.children) {
        return { ...item, title, children: loop(item.children) };
      }
      return { ...item, title };
    });
  };

  const linkToMenu = menu => {
    const { menuUrl } = menu;
    history.push(menuUrl);
  };

  const focusOutSearchInfo = e => {
    e.stopPropagation();
    let element = e.target;
    // 事件对象逐层网上找，只要不是菜单搜索信息内的就隐藏起来
    while (element && element !== page.current) {
      if (element === searchBox.current) return;
      element = element.parentNode;
    }
    setShowMenuInfo(false);
  };

  return (
    <div className={styles.commonMenu} ref={page} onClick={focusOutSearchInfo}>
      <Spin spinning={loading}>
        <div className={styles.searchWrapper}>
          <div className={styles.searchBox} ref={searchBox}>
            <div className={styles.searchTitle}>
              {/* <i className="iconfont">&#xe611;</i> */}
              <img
                style={{ width: '20px', marginRight: '0.5em' }}
                src={starIcon}
                alt=""
              />
              <span>我的常用菜单</span>
              <span>（{commonMenus.length}）</span>
            </div>
            <div className={styles.searchInput}>
              <Input
                maxLength={50}
                width={400}
                placeholder="搜索功能菜单"
                onChange={e => {
                  setShowMenuInfo(true);
                  setSearchInfo(e.target.value);
                }}
                onFocus={() => setShowMenuInfo(true)}
              />
            </div>
            <div
              className={classnames(
                styles.searchInfoBox,
                showMenuInfo ? '' : styles.searchInfoBox_hide,
              )}
            >
              <Tree
                height={400}
                showIcon
                defaultExpandAll
                defaultSelectedKeys={['0-0-0']}
                // switcherIcon={<DownOutlined />}
                treeData={menuList}
                titleRender={renderTitle}
                selectable={false}
              />
            </div>
          </div>
          {/* <Button className={styles.searchBtn}>添加菜单</Button> */}
        </div>
        <div className={styles.menuCardWrapper}>
          {commonMenus.map(item => (
            <MenuCard menu={item} linkToMenu={linkToMenu} key={item.menuID} />
          ))}
        </div>
        <div className={styles.pageLogo}>
          <img src={pageLogo} alt="" />
        </div>
      </Spin>
    </div>
  );
};

const MenuCard = ({ menu, linkToMenu }) => {
  const { menuIcon, menuName, menuGroup, menuID, menuUrl, menuPic } = menu;
  return (
    <div className={styles.menuCard} onClick={() => linkToMenu(menu)}>
      {/* <Link to={menuUrl} title={menuName}> */}
      <img className={styles.cardThumbnail} src={menuPic || thumbnail} alt="" />
      <div className={styles.cardLabel}>
        <div
          className={classnames(
            styles.cardTitle,
            isNeedFilterIcon(menuIcon) ? styles.filterIconBox : '',
          )}
        >
          <span className={styles.iconBox}>
            <img className={styles.cardIcon} src={menuIcon} alt="" />
          </span>
          <span className={styles.cardName}>{menuName}</span>
        </div>
        <div className={styles.cardGroup}>{menuGroup}</div>
      </div>
      {/* </Link> */}
    </div>
  );
};

const mapStateToProps = state => ({
  menus: state.getIn(['global', 'menu']),
});
export default connect(
  mapStateToProps,
  null,
)(CommonMenu);
